#!/usr/bin/env python2.3
#
#  slidingRegions.py
#  ENRAGE
#
import sys

print '%s: version 1.1' % sys.argv[0]

try:
    import psyco
    psyco.full()
except:
    print 'psyco not running'

from commoncode import *
from array import array

if len(sys.argv) < 6:
    print 'usage: python %s label windowSize rpkmThreshold rdsfile outfilename [-raw] [-nomulti] [-splices] [-cache pages]' % sys.argv[0]
    print '       where windowSize is in base pairs\n'
    sys.exit(1)

label = sys.argv[1]
windowSize = int(sys.argv[2])
threshold = float(sys.argv[3])
hitfilename = sys.argv[4]
outfilename = sys.argv[5]

doNormalize = True
withMulti = True
withSplices = False
doCache = False

if '-raw' in sys.argv:
    doNormalize = False

if '-nomulti' in sys.argv:
    withMulti = False

if '-splices' in sys.argv:
    withSplices = True

if '-cache' in sys.argv:
    doCache = True
    cachePages =  int(sys.argv[sys.argv.index('-cache') + 1])

hitRDS = readDataset(hitfilename, verbose = True, cache=doCache)

#sqlite default_cache_size is 2000 pages
if cachePages > hitRDS.getDefaultCacheSize():
    hitRDS.setDBcache(cachePages)

readlen = hitRDS.getReadSize()

normalizeBy = 1.
if doNormalize:
    normalizeBy = len(hitRDS) / 1000000.

chromList = hitRDS.getChromosomes()
chromList.sort()
halfWindow = int(windowSize / 2)
maxVal = 0
minVal = 0
index = 1
outfile = open(outfilename,'w')
outfile.write('#rdsfile = %s\traw = %s\n' % (hitfilename, not(doNormalize)))
outfile.write('# windowSize = %d bp\tthreshold = %.2f RPKM\n' % (windowSize, threshold))
outfile.write('#region\tchrom\tstart\tstop\tavgSig\n')

# converts from read coverage to RPKM over the window size
scale = 1000. / (readlen * windowSize)
print "scale %.2f" % scale
for achrom in chromList:
    print achrom
    lastNT = hitRDS.getMaxCoordinate(achrom, doMulti=withMulti, doSplices=withSplices) + readlen
    if lastNT <= 2 * windowSize:
        continue
    print "getting chromosome density profile"
    chromModel = hitRDS.getChromProfile(achrom, withMulti, withSplices, normalizeBy)
    
    previousVal = 0
    previousStart = 1
    chromIndex = 0
    inRegion = False
    regionBegin = 0
    currentVal = scale * sum(chromModel[:windowSize]) 
    print currentVal
    valueArray = array('f', windowSize * [currentVal])
    startPos = halfWindow
    if currentVal >= threshold:
        inRegion = True
    print "computing sliding window"
    for pos in xrange(startPos, lastNT - halfWindow - 1):
        if 18627535 <= pos <= 18627566:
            print currentVal, chromModel[pos], chromModel[pos - halfWindow], chromModel[pos + halfWindow], - scale * (chromModel[pos - halfWindow] + chromModel[pos + halfWindow]) 
        currentVal += scale * (chromModel[pos + halfWindow] - chromModel[pos - halfWindow]) 
        if currentVal < 0:
            currentVal = 0
        if currentVal >= threshold and not inRegion:
            regionBegin = pos
            inRegion = True
            valueArray = array('f', [currentVal])
        elif currentVal < threshold and inRegion:
            signal = sum(valueArray) / (pos - regionBegin)
            if not minVal <= signal <= maxVal:
                if signal > maxVal:
                    maxVal = signal
                    print "new maxlVal %.2f" % (maxVal)
            regionString = '%s%d\t%s\t%d\t%d\t%.2f\n' % (label, index, achrom, regionBegin, pos, signal)
            print regionString
            outfile.write(regionString)
            index += 1
            chromIndex += 1
            inRegion = False
        if inRegion:
            valueArray.append(currentVal)

    del chromModel
    if inRegion:
        signal = sum(valueArray) / (pos - regionBegin)
        if not minVal <= signal <= maxVal:
            if signal > maxVal:
                maxVal = signal
                print "new maxlVal %.2f" % (maxVal)
        regionString = '%s%d\t%s\t%d\t%d\t%.2f\n' % (label, index, achrom, regionBegin, pos, signal)
        print regionString
        outfile.write(regionString)
        index += 1
        chromIndex += 1
    print 'found %d regions' % chromIndex
outfile.close()
print "max value for sliding window %d is %.2f" % (windowSize, maxVal)
